package es.uva.tfg.geolocalizationgame.persistence;

import java.io.Serializable;
import java.util.List;

import javax.transaction.Transactional;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

import es.uva.tfg.geolocalizationgame.resources.Juego;
import es.uva.tfg.geolocalizationgame.resources.PuntoGeografico;

/**
 * Implementa la clase de persistencia para la entidad PuntoGeografico.
 * 
 * @author Cristina Gomez Escribano
 * @since 10/11/2018
 * @version 2.0
 *
 */
public interface PuntoGeograficoRepository extends JpaRepository<PuntoGeografico, Serializable> {
	
	/**
	 * Obtiene los puntos que pertenecen al juego pasado como parametro.
	 * 
	 * @param juego
	 * @return
	 */
	@Query("SELECT puntoGeografico FROM PuntoGeografico puntoGeografico WHERE puntoGeografico.juego=:juego")
	List<PuntoGeografico> getPuntosGeograficosByJuego(@Param(value="juego") Juego juego);
	
	/**
	 * Elimina de la base de datos todos los aciertos del punto pasado como parametro
	 * @param idPunto
	 */
	@Modifying
	@Transactional
	@Query(value = "DELETE FROM acierto where acierto.id_punto=:idPunto", nativeQuery = true)
	void deleteAciertos(@Param(value="idPunto") Integer idPunto);
	
	/**
	 * Elimina de la base de datos todos los fallos del punto pasado como parametro
	 * @param idPunto
	 */
	@Modifying
	@Transactional
	@Query(value = "DELETE FROM fallo where fallo.id_punto=:idPunto", nativeQuery = true)
	void deleteFallos(@Param(value="idPunto") Integer idPunto);
	
	/**
	 * Elimina de la base de datos todos los puntos conseguidos del punto pasado como parametro
	 * @param idPunto
	 */
	@Modifying
	@Transactional
	@Query(value = "DELETE FROM punto_conseguido where punto_conseguido.id_punto=:idPunto", nativeQuery = true)
	void deleteConseguidos(@Param(value="idPunto") Integer idPunto);

}
