package es.uva.tfg.geolocalizationgame.persistence;

import java.io.Serializable;
import java.util.List;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

import es.uva.tfg.geolocalizationgame.resources.EstadoSolicitudAmistad;
import es.uva.tfg.geolocalizationgame.resources.SolicitudAmistad;
import es.uva.tfg.geolocalizationgame.resources.Usuario;

/**
 * Contiene las operaciones de acceso a la base de datos para la entidad
 * SolicitudAmistad
 * 
 * @author Cristina Gómez Escribano
 * @version 2.0
 * @since 28/08/2018
 * 
 */

public interface SolicitudAmistadRepository extends JpaRepository<SolicitudAmistad, Serializable> {

	/**
	 * Obtiene todas las solicitudes recibidas de un usuario por login.
	 * 
	 * @param login
	 * @return List<SolicitudAmistad>
	 */
	@Query(value = "select * from solicitud_amistad solicitud where solicitud.RECEPTOR=:login", nativeQuery = true)
	List<SolicitudAmistad> getSolicitudes(@Param(value = "login") String login);

	/**
	 * Devuelve una solicitud entre dos usuarios por estado.
	 * 
	 * @param emisor
	 * @param receptor
	 * @param estado
	 * @return SolicitudAmistad
	 */
	@Query(value = "select * from solicitud_amistad solicitud where ((solicitud.RECEPTOR=:receptor and solicitud.EMISOR=:emisor) OR (solicitud.RECEPTOR=:emisor and solicitud.EMISOR=:receptor)) AND solicitud.ID_ESTADO_SOLICITUD=:estado", nativeQuery = true)
	SolicitudAmistad obtenerSolicitudByEmisorReceptorEstado(@Param(value = "emisor") String emisor,
			@Param(value = "receptor") String receptor, @Param(value = "estado") Integer estado);

	/**
	 * Devuelve las solicitudes enviadas de un usuario en un estado concreto.
	 * 
	 * @param usuario
	 * @param estado
	 * @return List<SolicitudAmistad>
	 */
	@Query("select solicitud from SolicitudAmistad solicitud where solicitud.emisor=:usuario and solicitud.estadoSolicitud=:estado")
	List<SolicitudAmistad> obtenerSolicitudesEnviadasByEstado(@Param(value = "usuario") Usuario usuario,
			@Param(value = "estado") EstadoSolicitudAmistad estado);
	
	/**
	 * Devuelve las solicitudes recibidas por un usuario en un estado concreto.
	 * 
	 * @param usuario
	 * @param estado
	 * @return List<SolicitudAmistad>
	 */
	@Query("select solicitud from SolicitudAmistad solicitud where solicitud.receptor=:usuario and solicitud.estadoSolicitud=:estado")
	List<SolicitudAmistad> obtenerSolicitudesRecibidasByEstado(@Param(value = "usuario") Usuario usuario,
			@Param(value = "estado") EstadoSolicitudAmistad estado);
}
