package es.uva.tfg.geolocalizationgame.resources;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import javax.validation.ConstraintViolation;

/**
 * Clase de utilidad que represtenda un mensaje de error.
 * 
 * @author Cristina Gomez Escribano
 * @version 1.0
 * @since 02/11/2018
 *
 */
public class ErrorMessage {

	private String error;
	private String description;

	public ErrorMessage() {

	}

	public ErrorMessage(Exception exception) {
		this(exception.getClass().getSimpleName(), exception.getMessage());
	}

	public ErrorMessage(String error, String description) {
		this.error = error;
		this.description = description;
	}

	public String getError() {
		return error;
	}

	public void setError(String error) {
		this.error = error;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}
	
	/**
	 * Convierte la lista de ConstraintViolations en una lista de ErrorMessage
	 * 
	 * @param constraintViolations
	 * @return lista de ErrorMessage
	 */
	public static List<ErrorMessage> getErrores(Set<ConstraintViolation<?>> constraintViolations) {
		/**
		 * En cada llamada a of genera un FieldError que lo agrega a la lista
		 */
		return constraintViolations.stream().map(ErrorMessage::convertirAFieldError).collect(Collectors.toList());
	}

	/**
	 * Convierte una ConstraintViolation en un FieldError
	 * 
	 * @return FieldError
	 */
	private static ErrorMessage convertirAFieldError(ConstraintViolation<?> constraintViolation) {

		String field = constraintViolation.getPropertyPath().toString();

		return new ErrorMessage(field, constraintViolation.getMessage());

	}

	@Override
	public String toString() {
		return "ErrorMessage [error=" + error + ", description=" + description + "]";
	}

}
