package es.uva.tfg.geolocalizationgame.resources;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;

import com.fasterxml.jackson.annotation.JsonView;

/**
 * Definicion de la entidad Juego.
 * 
 * @author Cristina Gomez Escribano
 * @version 6.0
 * @since 12/09/2018
 *
 */
@Entity
@Table(name = "juego")
public class Juego implements Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	@JsonView(DatosUser.PrivateProfile.class)
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@Column(name="ID_JUEGO", unique = true)
	private Integer idJuego;
	
	@JsonView({DatosUser.PrivateProfile.class, DatosUser.GameView.class})
	@NotNull
	@Size(min = 3, max = 30)
	@Pattern(regexp = "[ A-Za-zäÄëËïÏöÖüÜáéíóúáéíóúÁÉÍÓÚÂÊÎÔÛâêîôûàèìòùÀÈÌÒÙñÑ]+", message = "El nombre solo puede tener letrar, numeros y espacios")
	@Column(name="NOMBRE")
	private String nombre;
	
	@JsonView({DatosUser.PrivateProfile.class, DatosUser.GameView.class})
	@NotNull
	@JoinColumn(name="ID_TIPO_VISIBILIDAD", referencedColumnName="ID_TIPO_VISIBILIDAD", nullable=false)
	@ManyToOne(optional=false)
	private TipoVisibilidad visibilidad;
	
	@JsonView(DatosUser.GameView.class)
	@NotNull
	@JoinColumn(name="LOGIN_CREADOR")
	@ManyToOne
	private Usuario creador;
	
	@JsonView({DatosUser.PrivateProfile.class, DatosUser.GameView.class})
	@NotNull
	@Column(name="FECHA_CREACION")
	@Temporal(TemporalType.TIMESTAMP)
	private Date fechaCreacion;
	
	@JsonView(DatosUser.GameView.class)
	@NotNull
	@Column(name="FECHA_MODIFICACION")
	@Temporal(TemporalType.TIMESTAMP)
	private Date fechaModificacion;
	
	@JsonView(DatosUser.GameView.class)
	@OneToMany(mappedBy = "juego", cascade = CascadeType.ALL, fetch = FetchType.LAZY) 
	private List<Partida> partidas;
	
	@JsonView(DatosUser.GameView.class)
	@OneToMany(mappedBy = "juego", cascade = CascadeType.ALL, fetch = FetchType.LAZY) 
	private List<Comentario> comentarios;
	
	@JsonView(DatosUser.GameView.class)
	@OneToMany(mappedBy = "juego", cascade = CascadeType.ALL, fetch = FetchType.LAZY) 
	private List<PuntoGeografico> puntos;

	public Juego() {
		super();
	}

	public Juego(Integer idJuego, String nombre, TipoVisibilidad visibilidad, Usuario creador, Date fechaCreacion,
			Date fechaModificacion) {
		super();
		this.idJuego = idJuego;
		this.nombre = nombre;
		this.visibilidad = visibilidad;
		this.creador = creador;
		this.fechaCreacion = fechaCreacion;
		this.fechaModificacion = fechaModificacion;
	}

	public Integer getIdJuego() {
		return idJuego;
	}

	public void setIdJuego(Integer idJuego) {
		this.idJuego = idJuego;
	}

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public TipoVisibilidad getVisibilidad() {
		return visibilidad;
	}

	public void setVisibilidad(TipoVisibilidad visibilidad) {
		this.visibilidad = visibilidad;
	}

	public Usuario getCreador() {
		return creador;
	}

	public void setCreador(Usuario creador) {
		this.creador = creador;
	}

	public Date getFechaCreacion() {
		return fechaCreacion;
	}

	public void setFechaCreacion(Date fechaCreacion) {
		this.fechaCreacion = fechaCreacion;
	}

	public Date getFechaModificacion() {
		return fechaModificacion;
	}

	public void setFechaModificacion(Date fechaModificacion) {
		this.fechaModificacion = fechaModificacion;
	}

	public List<Partida> getPartidas() {
		return partidas;
	}

	public void setPartidas(List<Partida> partidas) {
		this.partidas = partidas;
	}

	public List<Comentario> getComentarios() {
		return comentarios;
	}

	public void setComentarios(List<Comentario> comentarios) {
		this.comentarios = comentarios;
	}

	public List<PuntoGeografico> getPuntos() {
		return puntos;
	}

	public void setPuntos(List<PuntoGeografico> puntos) {
		this.puntos = puntos;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((comentarios == null) ? 0 : comentarios.hashCode());
		result = prime * result + ((creador == null) ? 0 : creador.hashCode());
		result = prime * result + ((fechaCreacion == null) ? 0 : fechaCreacion.hashCode());
		result = prime * result + ((fechaModificacion == null) ? 0 : fechaModificacion.hashCode());
		result = prime * result + ((idJuego == null) ? 0 : idJuego.hashCode());
		result = prime * result + ((nombre == null) ? 0 : nombre.hashCode());
		result = prime * result + ((partidas == null) ? 0 : partidas.hashCode());
		result = prime * result + ((puntos == null) ? 0 : puntos.hashCode());
		result = prime * result + ((visibilidad == null) ? 0 : visibilidad.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Juego other = (Juego) obj;
		if (comentarios == null) {
			if (other.comentarios != null)
				return false;
		} else if (!comentarios.equals(other.comentarios))
			return false;
		if (creador == null) {
			if (other.creador != null)
				return false;
		} else if (!creador.equals(other.creador))
			return false;
		if (fechaCreacion == null) {
			if (other.fechaCreacion != null)
				return false;
		} else if (!fechaCreacion.equals(other.fechaCreacion))
			return false;
		if (fechaModificacion == null) {
			if (other.fechaModificacion != null)
				return false;
		} else if (!fechaModificacion.equals(other.fechaModificacion))
			return false;
		if (idJuego == null) {
			if (other.idJuego != null)
				return false;
		} else if (!idJuego.equals(other.idJuego))
			return false;
		if (nombre == null) {
			if (other.nombre != null)
				return false;
		} else if (!nombre.equals(other.nombre))
			return false;
		if (partidas == null) {
			if (other.partidas != null)
				return false;
		} else if (!partidas.equals(other.partidas))
			return false;
		if (puntos == null) {
			if (other.puntos != null)
				return false;
		} else if (!puntos.equals(other.puntos))
			return false;
		if (visibilidad == null) {
			if (other.visibilidad != null)
				return false;
		} else if (!visibilidad.equals(other.visibilidad))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Juego [idJuego=" + idJuego + ", nombre=" + nombre + ", visibilidad=" + visibilidad + ", creador="
				+ creador + ", fechaCreacion=" + fechaCreacion + ", fechaModificacion=" + fechaModificacion
				+ ", partidas=" + partidas + ", comentarios=" + comentarios + ", puntos=" + puntos + "]";
	}


	
}
