package es.uva.tfg.geolocalizationgame.resources;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;

import org.hibernate.validator.constraints.Email;

import com.fasterxml.jackson.annotation.JsonView;

/**
 * Implementa una entidad de Usuario
 * 
 * @author Cristina Gómez Escribano
 * @version 2.0
 * @since 02/05/2018
 *
 */
@Entity
@Table(name = "usuario")
public class Usuario implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@Id
	@Column(name = "LOGIN", unique = true)
	@Size(min = 3, max = 12)
	@Pattern(regexp = "[A-Za-z0-9nÑ]+", message = "El username puede contener letras y/o numeros")
	@JsonView({DatosUser.PrivateProfile.class, DatosUser.GameView.class})
	private String login;

	@NotNull
	@Column(name = "NOMBRE")
	@Size(min = 3, max = 30)
	@Pattern(regexp = "[ A-Za-zäÄëËïÏöÖüÜáéíóúáéíóúÁÉÍÓÚÂÊÎÔÛâêîôûàèìòùÀÈÌÒÙñÑ]+", message = "El nombre solo puede tener letras y espacios")
	@JsonView({DatosUser.PrivateProfile.class, DatosUser.GameView.class})
	private String nombre;

	@NotNull
	@Column(name = "PASSWORD")
	@Size(min = 7, max = 100)
	private String password;

	@NotNull
	@Column(name = "EMAIL")
	@Email
	@JsonView(DatosUser.PrivateProfile.class)
	private String email;

	@NotNull
	@Column(name = "AVATAR")
	@JsonView({DatosUser.PrivateProfile.class, DatosUser.GameView.class})
	private String avatar;

	@OneToMany(mappedBy = "emisor")
	private List<SolicitudAmistad> solicitudesEnviadas;

	@OneToMany(mappedBy = "receptor")
	private List<SolicitudAmistad> solicitudesRecibidas;

	@ManyToMany
	@JoinTable(name = "amistad", joinColumns = @JoinColumn(name = "LOGIN"), inverseJoinColumns = @JoinColumn(name = "LOGIN_AMIGO"))
	private List<Usuario> amigos;

	@ManyToMany
	@JoinTable(name = "amistad", joinColumns = @JoinColumn(name = "LOGIN_AMIGO"), inverseJoinColumns = @JoinColumn(name = "LOGIN"))
	private List<Usuario> amigosDe;
	
	@JsonView(DatosUser.PrivateProfile.class)
	@OneToMany(mappedBy = "creador", cascade = CascadeType.ALL, fetch = FetchType.LAZY) 
	private List<Juego> juegos;

	@JsonView(DatosUser.Basic.class)
	@OneToMany(mappedBy = "creador", cascade = CascadeType.ALL, fetch = FetchType.LAZY) 
	private List<Partida> partidasJugadas;
	
	@JsonView(DatosUser.Basic.class)
	@OneToMany(mappedBy = "creador", cascade = CascadeType.ALL, fetch = FetchType.LAZY) 
	private List<Comentario> comentariosCreados;
	
	@JsonView(DatosUser.PrivateProfile.class)
	@OneToMany(mappedBy = "id.usuario", cascade = CascadeType.ALL, fetch = FetchType.LAZY) 
	private List<Logro> logros;
	
	@ManyToMany
	@JoinTable(name = "punto_conseguido", joinColumns = @JoinColumn(name = "LOGIN"), inverseJoinColumns = @JoinColumn(name = "ID_PUNTO"))
	private List<PuntoGeografico> puntosConseguidos;

	
	public Usuario() {
		super();
	}

	public Usuario(String login, String nombre, String password, String email, Date fechaDeNacimiento, String avatar) {
		super();
		this.login = login;
		this.nombre = nombre;
		this.password = password;
		this.email = email;
		this.avatar = avatar;
	}

	public String getLogin() {
		return login;
	}

	public void setLogin(String login) {
		this.login = login;
	}

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getAvatar() {
		return avatar;
	}

	public void setAvatar(String avatar) {
		this.avatar = avatar;
	}

	public List<SolicitudAmistad> getSolicitudesEnviadas() {
		return solicitudesEnviadas;
	}

	public void setSolicitudesEnviadas(List<SolicitudAmistad> solicitudesEnviadas) {
		this.solicitudesEnviadas = solicitudesEnviadas;
	}

	public List<SolicitudAmistad> getSolicitudesRecibidas() {
		return solicitudesRecibidas;
	}

	public void setSolicitudesRecibidas(List<SolicitudAmistad> solicitudesRecibidas) {
		this.solicitudesRecibidas = solicitudesRecibidas;
	}

	public List<Usuario> getAmigos() {
		return amigos;
	}

	public void setAmigos(List<Usuario> amigos) {
		this.amigos = amigos;
	}

	public List<Usuario> getAmigosDe() {
		return amigosDe;
	}

	public void setAmigosDe(List<Usuario> amigosDe) {
		this.amigosDe = amigosDe;
	}

	public List<Juego> getJuegos() {
		return juegos;
	}

	public void setJuegos(List<Juego> juegos) {
		this.juegos = juegos;
	}

	public List<Partida> getPartidasJugadas() {
		return partidasJugadas;
	}

	public void setPartidasJugadas(List<Partida> partidasJugadas) {
		this.partidasJugadas = partidasJugadas;
	}

	public List<Comentario> getComentariosCreados() {
		return comentariosCreados;
	}

	public void setComentariosCreados(List<Comentario> comentariosCreados) {
		this.comentariosCreados = comentariosCreados;
	}

	public List<Logro> getLogros() {
		return logros;
	}

	public void setLogros(List<Logro> logros) {
		this.logros = logros;
	}

	public List<PuntoGeografico> getPuntosConseguidos() {
		return puntosConseguidos;
	}

	public void setPuntosConseguidos(List<PuntoGeografico> puntosConseguidos) {
		this.puntosConseguidos = puntosConseguidos;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((amigos == null) ? 0 : amigos.hashCode());
		result = prime * result + ((amigosDe == null) ? 0 : amigosDe.hashCode());
		result = prime * result + ((avatar == null) ? 0 : avatar.hashCode());
		result = prime * result + ((comentariosCreados == null) ? 0 : comentariosCreados.hashCode());
		result = prime * result + ((email == null) ? 0 : email.hashCode());
		result = prime * result + ((juegos == null) ? 0 : juegos.hashCode());
		result = prime * result + ((login == null) ? 0 : login.hashCode());
		result = prime * result + ((logros == null) ? 0 : logros.hashCode());
		result = prime * result + ((nombre == null) ? 0 : nombre.hashCode());
		result = prime * result + ((partidasJugadas == null) ? 0 : partidasJugadas.hashCode());
		result = prime * result + ((password == null) ? 0 : password.hashCode());
		result = prime * result + ((puntosConseguidos == null) ? 0 : puntosConseguidos.hashCode());
		result = prime * result + ((solicitudesEnviadas == null) ? 0 : solicitudesEnviadas.hashCode());
		result = prime * result + ((solicitudesRecibidas == null) ? 0 : solicitudesRecibidas.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Usuario other = (Usuario) obj;
		if (amigos == null) {
			if (other.amigos != null)
				return false;
		} else if (!amigos.equals(other.amigos))
			return false;
		if (amigosDe == null) {
			if (other.amigosDe != null)
				return false;
		} else if (!amigosDe.equals(other.amigosDe))
			return false;
		if (avatar == null) {
			if (other.avatar != null)
				return false;
		} else if (!avatar.equals(other.avatar))
			return false;
		if (comentariosCreados == null) {
			if (other.comentariosCreados != null)
				return false;
		} else if (!comentariosCreados.equals(other.comentariosCreados))
			return false;
		if (email == null) {
			if (other.email != null)
				return false;
		} else if (!email.equals(other.email))
			return false;
		if (juegos == null) {
			if (other.juegos != null)
				return false;
		} else if (!juegos.equals(other.juegos))
			return false;
		if (login == null) {
			if (other.login != null)
				return false;
		} else if (!login.equals(other.login))
			return false;
		if (logros == null) {
			if (other.logros != null)
				return false;
		} else if (!logros.equals(other.logros))
			return false;
		if (nombre == null) {
			if (other.nombre != null)
				return false;
		} else if (!nombre.equals(other.nombre))
			return false;
		if (partidasJugadas == null) {
			if (other.partidasJugadas != null)
				return false;
		} else if (!partidasJugadas.equals(other.partidasJugadas))
			return false;
		if (password == null) {
			if (other.password != null)
				return false;
		} else if (!password.equals(other.password))
			return false;
		if (puntosConseguidos == null) {
			if (other.puntosConseguidos != null)
				return false;
		} else if (!puntosConseguidos.equals(other.puntosConseguidos))
			return false;
		if (solicitudesEnviadas == null) {
			if (other.solicitudesEnviadas != null)
				return false;
		} else if (!solicitudesEnviadas.equals(other.solicitudesEnviadas))
			return false;
		if (solicitudesRecibidas == null) {
			if (other.solicitudesRecibidas != null)
				return false;
		} else if (!solicitudesRecibidas.equals(other.solicitudesRecibidas))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Usuario [login=" + login + ", nombre=" + nombre + ", password=" + password + ", email=" + email
				+ ", avatar=" + avatar + ", solicitudesEnviadas=" + solicitudesEnviadas + ", solicitudesRecibidas="
				+ solicitudesRecibidas + ", amigos=" + amigos + ", amigosDe=" + amigosDe + ", juegos=" + juegos
				+ ", partidasJugadas=" + partidasJugadas + ", comentariosCreados=" + comentariosCreados + ", logros="
				+ logros + ", puntosConseguidos=" + puntosConseguidos + "]";
	}
}
