package es.uva.tfg.geolocalizationgame.services;

import java.util.List;

import javax.validation.Valid;

import org.json.JSONException;
import org.json.JSONObject;

import es.uva.tfg.geolocalizationgame.exceptions.TipoPuntoNotFoundException;
import es.uva.tfg.geolocalizationgame.resources.Juego;
import es.uva.tfg.geolocalizationgame.resources.PuntoGeografico;

/**
 * Implementa la clase de servicio para la entidad PuntoGeografico.
 * 
 * @author Cristina Gomez Escribano
 * @since 10/11/2018
 * @version 3.0
 */
public interface PuntoGeograficoService {

	/**
	 * Crea un objeto punto a partir de un json de entrada
	 * @param punto
	 * @return puntoGeografico
	 * @throws JSONException 
	 * @throws TipoPuntoNotFoundException 
	 */
	PuntoGeografico crearPunto(JSONObject punto, @Valid Juego juego) throws JSONException, TipoPuntoNotFoundException;
	
	/**
	 * Devuelve un punto geografico por id.
	 * 
	 * @param idPunto
	 * @return PuntoGeografico
	 */
	PuntoGeografico obtenerPuntoGeograficoById(Integer idPunto);
	
	/**
	 * Elimina un punto geografico.
	 *  
	 * @param punto
	 */
	void eliminarPuntoGeografico(@Valid PuntoGeografico punto);

	/**
	 * Devuelve la lista de puntos geograficos de un juego.
	 * 
	 * @param juego
	 * @return
	 */
	List<PuntoGeografico> obtenerPuntosGeograficosByJuego(@Valid Juego juego);
	
	/**
	 * Realiza la operacion para guardar un punto en el repositorio.
	 * 
	 * @param punto
	 */
	void savePunto(@Valid PuntoGeografico punto);

	/**
	 * Comprueba si existe, en el juego, el punto geografico pasado como paramentro.
	 * 
	 * @param punto
	 * @return true or false
	 */
	boolean existPuntoGeografico(@Valid Juego juego, @Valid PuntoGeografico punto);
}
