package es.uva.tfg.geolocalizationgame.services;

import java.util.List;

import javax.validation.Valid;

import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import es.uva.tfg.geolocalizationgame.exceptions.TipoPuntoNotFoundException;
import es.uva.tfg.geolocalizationgame.persistence.PuntoGeograficoRepository;
import es.uva.tfg.geolocalizationgame.resources.Juego;
import es.uva.tfg.geolocalizationgame.resources.PuntoGeografico;
import es.uva.tfg.geolocalizationgame.resources.TipoPunto;

/**
 * Implementacion del servicio para la entidad PuntoGeogradico.
 * 
 * @author Cristina Gomez Escribano
 * @since 10/11/2018
 * @version 3.0
 */
@Service
public class PuntoGeograficoServiceImpl implements PuntoGeograficoService {

	@Autowired
	private TipoPuntoService tipoPuntoService;
	
	@Autowired
	private PuntoGeograficoRepository puntoRepository;
	
	@Override
	public PuntoGeografico crearPunto(JSONObject punto, @Valid Juego juego) throws JSONException, TipoPuntoNotFoundException {
		String nombre=punto.getString("nombre");
		Double latitud=punto.getDouble("latitud");
		Double longitud=punto.getDouble("longitud");
		String pais=punto.getString("pais");
		String continente=punto.getString("continente");
		String tipoPunto=punto.getString("tipoPunto");
		TipoPunto tipo=tipoPuntoService.obtenerTipoPuntoByNombre(tipoPunto);
		if(tipo==null) {
			throw new TipoPuntoNotFoundException();
		}
		PuntoGeografico puntoGeografico=new PuntoGeografico();
		puntoGeografico.setNombre(nombre);
		puntoGeografico.setLatitud(latitud);
		puntoGeografico.setLongitud(longitud);
		puntoGeografico.setPais(pais);
		puntoGeografico.setContinente(continente);
		puntoGeografico.setIdTipoPunto(tipo);
		puntoGeografico.setJuego(juego);
		return puntoGeografico;
	}

	@Override
	public PuntoGeografico obtenerPuntoGeograficoById(Integer idPunto) {
		return puntoRepository.findOne(idPunto);
	}

	@Override
	public void eliminarPuntoGeografico(@Valid PuntoGeografico punto) {
		puntoRepository.deleteAciertos(punto.getIdPunto());
		puntoRepository.deleteFallos(punto.getIdPunto());
		puntoRepository.deleteConseguidos(punto.getIdPunto());
		puntoRepository.delete(punto);
	}

	@Override
	public List<PuntoGeografico> obtenerPuntosGeograficosByJuego(@Valid Juego juego) {
		return puntoRepository.getPuntosGeograficosByJuego(juego);
	}

	@Override
	public void savePunto(@Valid PuntoGeografico punto) {
		puntoRepository.saveAndFlush(punto);
	}

	@Override
	public boolean existPuntoGeografico(@Valid Juego juego, @Valid PuntoGeografico punto) {
		boolean exist=false;
		List<PuntoGeografico> puntos = obtenerPuntosGeograficosByJuego(juego);
		for (int i=0; i<puntos.size(); i++) {
			if(puntos.get(i).getLatitud().equals(punto.getLatitud()) && puntos.get(i).getLongitud().equals(punto.getLongitud())){
				exist=true;
			}
		}
		return exist;
	}
}
