package es.uva.tfg.geolocalizationgame.services;

import java.util.List;

import javax.validation.Valid;

import es.uva.tfg.geolocalizationgame.resources.MapeoUser;
import es.uva.tfg.geolocalizationgame.resources.PuntoGeografico;
import es.uva.tfg.geolocalizationgame.resources.Usuario;

/**
 * Definición de las firmas de lo métodos disponibles en el servicio para las transacciones de la entidad usuario
 * 
 * @author Cristina Gómez Escribano
 * @version 3.0
 * @since 02/05/2018
 *
 */
public interface UserService {

	/**
	 * Permite crear un nuevo usuario en el sistema
	 * 
	 * @param usuario
	 * @return True si se ha creado o False si ya existía
	 */
	boolean altaUsuario(@Valid Usuario usuario);

	/**
	 * Devuelve un usuario por login
	 * @param usuario
	 * @return Usuario o null si no existe
	 */
	Usuario obtenerUsuarioByLogin(String login);

	/**
	 * Devuelve el usuario actualizado en la base de datos
	 * @param usuario
	 * @return Usuario o null si no se ha podido actualizar
	 */
	Usuario updateUserData(@Valid Usuario usuario);

	/**
	 * Actualiza los datos del usuario.
	 * 
	 * @param usuario
	 * @param userCurrent
	 * @return boolean
	 */
	boolean actualizarDatosUsuario(MapeoUser usuario, Usuario userCurrent);
	
	/**
	 * Devuelve la lista de usuarios que contengan el texto
	 * @param text
	 * @return lista de usuarios
	 */
	List<Usuario> getUsuarios(String text);

	/**
	 * Actualiza los amigos de un usuario con el nuevo amigo
	 * 
	 * @param usuario
	 * @param newAmigo
	 * @return true or false
	 */
	boolean agregarAmigo(@Valid Usuario usuario, @Valid Usuario newAmigo);
	
	/**
	 * Agrega los nuevos puntos conseguido a la lista de puntos conseguidos del usuario, si no lo tiene ya.
	 * 
	 * @param usuario
	 * @param puntosNuevos
	 */
	void actualizarPuntosConseguidos(@Valid Usuario usuario, List<PuntoGeografico> puntosNuevos);
	
	/**
	 * Elimina un amigo de un usuario.
	 * 
	 * @param usuario
	 * @param newAmigo
	 * @return boolean
	 */
	boolean eliminarAmigo(@Valid Usuario usuario, @Valid Usuario newAmigo);

	/**
	 * Actualiza los logros de los amigos de un usuario.
	 * 
	 * @param usuario
	 */
	void actualizarLogrosAmigos(@Valid Usuario usuario);

	/**
	 * Actualiza los logros del usuario.
	 * 
	 * @param usuario
	 */
	void actualizarLogrosUsuario(@Valid Usuario usuario);

	/**
	 * Actualiza los juegos de los amigos.
	 * 
	 * @param usuario
	 */
	void actualizarJuegosAmigos(@Valid Usuario usuario);

	/**
	 * Actualiza los juegos del usuario.
	 * 
	 * @param usuario
	 */
	void actualizarJuegosUsuario(@Valid Usuario usuario);
}
