app.controller('NavbarController', function($window, $scope, $rootScope, $location, $translate, UserService, SolicitudAmistadService, JuegoService, BusquedaService, TranslateService) {
	
	$scope.$emit('LOAD');
	$rootScope.portada=false;
	$rootScope.user=UserService.getUser();
	$scope.textToFind="";
	isAuth();
	var language;
	if(localStorage.getItem("idioma")){
		language = localStorage.getItem("idioma").substr(0,localStorage.getItem("idioma").indexOf("-"));
	}else{
		language=($window.navigator.userLanguage || $window.navigator.language).indexOf("en") == 0 ? "en":"es";

	}

	obtenerJsonLanguage(language);
	
	/**
	 * Realiza el cambio de idioma al seleccionarlo
	 */
	$scope.changeLanguage = function (language){
		var nameJson = language.indexOf("en") == 0 ? "en":"es";
		$translate.use(language);
		obtenerJsonLanguage(nameJson);
		localStorage.setItem("idioma",language);
	};
	
	/**
	 * Cerrar la barra de navegacion al pulsar en una opcion del menu
	 */
	$(".close-nav").click(function(){
	    if($(".navbar-toggler").attr("aria-expanded")==="true"){//expanded navbar
	    	$("button[class='navbar-toggler']").click();
	    }
	});
	
	/**
	 * Define le home, dependiendo de si está logado o no
	 */
	$scope.goHome = function () {
		if(UserService.userAuth){
			$location.path("/home");
		}else{
			$location.path("/");
		}
	};
	
	function obtenerJsonLanguage(language){
		TranslateService.obtenerPaisesContinentes(language)
		.then(function(response){
			  $rootScope.paises= response.countries;
			  $rootScope.continentes=response.continents;
			  $rootScope.tipoPunto=response.typePoint;
		  }, function error(error){
			  $rootScope.paises=[];
			  $rootScope.continentes=[];
			  $rootScope.tipoPunto= [];
		  });
	}
	

	/**
	 * Gestiona la búsqueda de un texto en la barra de busqueda
	 */
	$scope.textSearch = function (){
		BusquedaService.searchUsersAndGamesByText($scope.textToFind)
		.then(function success(response){
			$rootScope.search=true;
			$rootScope.usuarios=UserService.getSearchUsers();
			$rootScope.juegos=JuegoService.getSearchJuegos();
			scrollToTop();
			$location.path("/home/resultSearch");
			
		}, function error(error){
			$rootScope.search=true;
			$rootScope.usuarios={};
			$rootScope.juegos={};
			scrollToTop();
			$location.path("/home/resultSearch");
		});
	
	};
	
	/**
	 * Comprueba si el usuario está logado
	 */
	function isAuth() {
		if($location.path().indexOf("/home") > -1){//esta en url de /home, por tanto autenticado
			$rootScope.authenticated = true;
			if(!UserService.getUser()){ //se ha recargado la pagina, volver a obtener el usuario
				UserService.getUserAuth()
				.then(function success(response) {
					if (response.login) { //esta logueado
						UserService.userAuth=true;
						$rootScope.user=UserService.getUser(); //guardar el usuario logado
						obtenerSolicitudes();
						obtenerAmigos();
					} else {
						$rootScope.authenticated = false;
						$location.path("/");
					}
					$scope.$emit('UNLOAD');
				}, function error (error){
					$rootScope.authenticated = false;
					$location.path("/");
					$scope.$emit('UNLOAD');
				});
			}
		}else{
			$rootScope.authenticated = false;
			$scope.$emit('UNLOAD');
		}
	};
	
	function obtenerSolicitudes () {
		SolicitudAmistadService.obtenerSolicitudesEnviadas($rootScope.user.login, "Pendiente").then(
			function success(response){
				$rootScope.user.solicitudesEnviadas=response;
				$rootScope.loading=false;
			},
			function error(error){
				$rootScope.user.solicitudesEnviadas={};
			}
		);
		SolicitudAmistadService.obtenerSolicitudesRecibidas($rootScope.user.login, "Pendiente").then(
				function success(response){
					$rootScope.user.solicitudesRecibidas=response;
					
				},
				function error(error){
					$rootScope.user.solicitudesRecibidas={};
				}
			);
	};
	
	function obtenerAmigos() {
		UserService.getAmigos($rootScope.user.login).then(
			function success(response) {
				$rootScope.user.amigos = response;
				obtenerLogrosCompartidos();
				obtenerJuegosCompartidos();
			}, function error(error) {
				$rootScope.user.amigos = {};
			});

	}
	;
	
	/**
	 * Comprueba si la url que se le pasa como parámetro es igual a la actual
	 */
	$scope.isActive = function (viewLocation){
		return viewLocation === $location.path();
	};
	
	/**
	 * Cierre de sesión de la aplicación
	 */
	$scope.logout = function() {
		UserService.logoutUser()
		.then(function success(response) {
			$rootScope.authenticated = false;
			$rootScope.user={};
			UserService.userAuth=false;
			$scope.textToFind="";
			$location.path("/");
		},
		function error(error) {
			$rootScope.authenticated = false;
			$rootScope.user={};
			UserService.userAuth=false;
			$scope.textToFind="";
		});
	}
	
	/**
	 * Guarda los logros compartidos por los amigos con visibilidad amigos
	 */
	function obtenerLogrosCompartidos(){
		$rootScope.user.logrosCompartidos=[];
		jQuery.each($rootScope.user.amigos, function(i, amigo) {
			jQuery.each(amigo.logros, function(j, logro) {
				if (logro.tipoVisibilidad.tipoVisibilidad === "amigos") {
					logro.usuario=amigo;
					$rootScope.user.logrosCompartidos.push(logro);
				}
			});
		});
	};
	
	/**
	 * Guarda los juegos compartidos por los amigos con visibilidad amigos
	 */
	function obtenerJuegosCompartidos(){
		$rootScope.user.juegosCompartidos=[];
		jQuery.each($rootScope.user.amigos, function(i, amigo) {
			jQuery.each(amigo.juegos, function(j, juego) {
				if (juego.visibilidad.tipoVisibilidad === "amigos") {
					juego.usuario=amigo;
					$rootScope.user.juegosCompartidos.push(juego);
				}
			});
		});
	};
});