CREATE DATABASE `dbgeolocalizationgame` CHARACTER SET utf8 COLLATE utf8_bin;
USE dbgeolocalizationgame;

CREATE TABLE MODALIDAD_JUEGO (
	ID_MODALIDAD INTEGER,
	MODALIDAD VARCHAR (15) NOT NULL UNIQUE,
	PRIMARY KEY (ID_MODALIDAD)
);

CREATE TABLE NIVEL_DIFICULTAD(
	ID_NIVEL_DIFICULTAD INTEGER,
	NIVEL_DIFICULTAD VARCHAR(10) NOT NULL UNIQUE,
	PRIMARY KEY (ID_NIVEL_DIFICULTAD)
);

CREATE TABLE TIPO_VISIBILIDAD (
	ID_TIPO_VISIBILIDAD INTEGER,
	TIPO_VISIBILIDAD VARCHAR(10) NOT NULL UNIQUE,
	PRIMARY KEY (ID_TIPO_VISIBILIDAD)
);

CREATE TABLE TIPO_PUNTO (
	ID_TIPO_PUNTO INTEGER,
	TIPO_PUNTO VARCHAR(15) NOT NULL UNIQUE,
	PRIMARY KEY (ID_TIPO_PUNTO)
);

CREATE TABLE ESTADO_SOLICITUD_AMISTAD ( 
	ID_ESTADO_SOLICITUD INTEGER,
	ESTADO VARCHAR (10) NOT NULL UNIQUE,
	PRIMARY KEY (ID_ESTADO_SOLICITUD)
);

CREATE TABLE USUARIO (
	LOGIN VARCHAR(12),
	NOMBRE VARCHAR(30) NOT NULL,
	PASSWORD VARCHAR (100) NOT NULL,
	EMAIL VARCHAR (30) NOT NULL,
	AVATAR VARCHAR (1) NOT NULL,
	PRIMARY KEY (LOGIN)
);

CREATE TABLE SOLICITUD_AMISTAD (
	ID_SOLICITUD_AMISTAD INTEGER,
	ID_ESTADO_SOLICITUD INTEGER NOT NULL,
	FECHA_ENVIO TIMESTAMP NOT NULL,
	FECHA_MODIFICACION TIMESTAMP NOT NULL,
	EMISOR VARCHAR (12) NOT NULL,
	RECEPTOR VARCHAR (12) NOT NULL,
	PRIMARY KEY (ID_SOLICITUD_AMISTAD),
	FOREIGN KEY (ID_ESTADO_SOLICITUD) REFERENCES ESTADO_SOLICITUD_AMISTAD(ID_ESTADO_SOLICITUD),
	FOREIGN KEY (EMISOR) REFERENCES USUARIO (LOGIN) ON DELETE CASCADE,
	FOREIGN KEY (RECEPTOR) REFERENCES USUARIO (LOGIN) ON DELETE CASCADE
);

CREATE TABLE AMISTAD(
	LOGIN VARCHAR (12),
	LOGIN_AMIGO VARCHAR (12),
	PRIMARY KEY (LOGIN, LOGIN_AMIGO),
	FOREIGN KEY (LOGIN) REFERENCES USUARIO(LOGIN) ON DELETE CASCADE,
	FOREIGN KEY (LOGIN_AMIGO) REFERENCES USUARIO(LOGIN) ON DELETE CASCADE
);

CREATE TABLE JUEGO (
	ID_JUEGO INTEGER,
	NOMBRE VARCHAR (30) NOT NULL,
	ID_TIPO_VISIBILIDAD INTEGER NOT NULL,
	LOGIN_CREADOR VARCHAR (12) NOT NULL,
	FECHA_CREACION TIMESTAMP NOT NULL,
	FECHA_MODIFICACION TIMESTAMP NOT NULL,
	PRIMARY KEY (ID_JUEGO),
	FOREIGN KEY (ID_TIPO_VISIBILIDAD) REFERENCES TIPO_VISIBILIDAD (ID_TIPO_VISIBILIDAD),
	FOREIGN KEY (LOGIN_CREADOR) REFERENCES USUARIO (LOGIN) ON DELETE CASCADE
);

CREATE TABLE PUNTO_GEOGRAFICO (
	ID_PUNTO INTEGER,
	NOMBRE VARCHAR(200) NOT NULL,
	LATITUD REAL NOT NULL,
	LONGITUD REAL NOT NULL,
	PAIS VARCHAR (30) NOT NULL,
	CONTINENTE VARCHAR (20) NOT NULL,
	ID_TIPO_PUNTO INTEGER NOT NULL,
	ID_JUEGO INTEGER NOT NULL,
 	PRIMARY KEY (ID_PUNTO),
	FOREIGN KEY (ID_JUEGO) REFERENCES JUEGO (ID_JUEGO),
	FOREIGN KEY (ID_TIPO_PUNTO) REFERENCES TIPO_PUNTO (ID_TIPO_PUNTO)
);

CREATE TABLE PUNTO_CONSEGUIDO (
	ID_PUNTO INTEGER,
	LOGIN VARCHAR (12),
	PRIMARY KEY (ID_PUNTO, LOGIN),
	FOREIGN KEY (ID_PUNTO) REFERENCES PUNTO_GEOGRAFICO (ID_PUNTO),
	FOREIGN KEY (LOGIN) REFERENCES USUARIO (LOGIN)
);

CREATE TABLE PARTIDA (
	ID_PARTIDA INTEGER,
	PUNTUACION REAL NOT NULL,
	ID_NIVEL_DIFICULTAD INTEGER NOT NULL,
	ID_MODALIDAD INTEGER NOT NULL,
	FECHA_PARTIDA TIMESTAMP NOT NULL,
	ID_JUEGO INTEGER NOT NULL,
	LOGIN VARCHAR (12) NOT NULL,	
	PRIMARY KEY (ID_PARTIDA),
	FOREIGN KEY (ID_NIVEL_DIFICULTAD) REFERENCES NIVEL_DIFICULTAD (ID_NIVEL_DIFICULTAD),
	FOREIGN KEY (ID_MODALIDAD) REFERENCES MODALIDAD_JUEGO (ID_MODALIDAD),
	FOREIGN KEY (ID_JUEGO) REFERENCES JUEGO (ID_JUEGO),
	FOREIGN KEY (LOGIN) REFERENCES USUARIO (LOGIN)
);

CREATE TABLE ACIERTO (
	ID_PARTIDA INTEGER,
	ID_PUNTO INTEGER,
	PRIMARY KEY (ID_PARTIDA, ID_PUNTO),
	FOREIGN KEY (ID_PARTIDA) REFERENCES PARTIDA (ID_PARTIDA),
	FOREIGN KEY (ID_PUNTO) REFERENCES PUNTO_GEOGRAFICO (ID_PUNTO)
);

CREATE TABLE FALLO (
	ID_PARTIDA INTEGER,
	ID_PUNTO INTEGER,
	PRIMARY KEY (ID_PARTIDA, ID_PUNTO),
	FOREIGN KEY (ID_PARTIDA) REFERENCES PARTIDA (ID_PARTIDA),
	FOREIGN KEY (ID_PUNTO) REFERENCES PUNTO_GEOGRAFICO (ID_PUNTO)
);

CREATE TABLE TIPO_LOGRO (
	ID_TIPO_LOGRO INTEGER,
	LOGRO VARCHAR(30) NOT NULL UNIQUE,
	DESCRIPCION VARCHAR (100) NOT NULL,
	PUNTOS_A_CONSEGUIR INTEGER NOT NULL,
	ID_TIPO_PUNTO INTEGER,
	CONTINENTE VARCHAR (20),
	PAIS VARCHAR (30),
	PRIMARY KEY (ID_TIPO_LOGRO),
	FOREIGN KEY (ID_TIPO_PUNTO) REFERENCES TIPO_PUNTO (ID_TIPO_PUNTO)
);

CREATE TABLE LOGRO (
	LOGIN_LOGRO VARCHAR (12),
	ID_TIPO_LOGRO INTEGER,
	FECHA_CONSEGUIDO TIMESTAMP NOT NULL,
	FECHA_MODIFICACION TIMESTAMP NOT NULL,
	ID_TIPO_VISIBILIDAD INTEGER NOT NULL,
	PRIMARY KEY (LOGIN_LOGRO, ID_TIPO_LOGRO),
	FOREIGN KEY (LOGIN_LOGRO) REFERENCES USUARIO (LOGIN) ON DELETE CASCADE,
	FOREIGN KEY (ID_TIPO_LOGRO) REFERENCES TIPO_LOGRO (ID_TIPO_LOGRO) ON DELETE CASCADE,
	FOREIGN KEY (ID_TIPO_VISIBILIDAD) REFERENCES TIPO_VISIBILIDAD (ID_TIPO_VISIBILIDAD)
);

CREATE TABLE COMENTARIO (
	ID_COMENTARIO INTEGER,
	DESCRIPCION VARCHAR (150) NOT NULL,
	FECHA_EMISION TIMESTAMP NOT NULL,
	FECHA_MODIFICACION TIMESTAMP NOT NULL,
	ID_JUEGO INTEGER,
	LOGIN_CREADOR VARCHAR (12) NOT NULL,
	PRIMARY KEY (ID_COMENTARIO),
	FOREIGN KEY (LOGIN_CREADOR) REFERENCES USUARIO (LOGIN) ON DELETE CASCADE,
	FOREIGN KEY (ID_JUEGO) REFERENCES JUEGO (ID_JUEGO) ON DELETE CASCADE
);